@@ -1,9 +1,9 @@
 /*-------------------------------------------------------------------------
 *
-* Copyright (c) 2003-2008, PostgreSQL Global Development Group
+* Copyright (c) 2003-2011, PostgreSQL Global Development Group
 *
 * IDENTIFICATION
-*   $PostgreSQL: pgjdbc/org/postgresql/jdbc2/AbstractJdbc2ResultSet.java,v 1.101.2.2 2009/04/19 16:14:46 jurka Exp $
+*   $PostgreSQL: pgjdbc/org/postgresql/jdbc2/AbstractJdbc2ResultSet.java,v 1.112 2011/08/02 13:48:35 davecramer Exp $
 *
 *-------------------------------------------------------------------------
 */
@@ -32,6 +32,7 @@ import org.postgresql.util.PGtokenizer;
 import org.postgresql.util.PSQLException;
 import org.postgresql.util.PSQLState;
 import org.postgresql.util.GT;
+import org.postgresql.PGResultSetMetaData;
 
 
 public abstract class AbstractJdbc2ResultSet implements BaseResultSet, org.postgresql.PGRefCursorResultSet
@@ -50,8 +51,8 @@ public abstract class AbstractJdbc2ResultSet implements BaseResultSet, org.postg
     private PreparedStatement insertStatement = null;
     private PreparedStatement deleteStatement = null;
     private PreparedStatement selectStatement = null;
-    private int resultsettype;
-    private int resultsetconcurrency;
+    private final int resultsettype;
+    private final int resultsetconcurrency;
     private int fetchdirection = ResultSet.FETCH_UNKNOWN;
     protected final BaseConnection connection;  // the connection we belong to
     protected final BaseStatement statement;    // the statement we belong to
@@ -74,7 +75,7 @@ public abstract class AbstractJdbc2ResultSet implements BaseResultSet, org.postg
     protected boolean wasNullFlag = false;
     protected boolean onInsertRow = false;  // are we on the insert row (for JDBC2 updatable resultsets)?
 
-    public byte[][] rowBuffer = null;       // updateable rowbuffer
+    private byte[][] rowBuffer = null;       // updateable rowbuffer
 
     protected int fetchSize;       // Current fetch size (might be 0).
     protected ResultCursor cursor; // Cursor for fetching additional data.
@@ -162,6 +163,9 @@ public abstract class AbstractJdbc2ResultSet implements BaseResultSet, org.postg
             if (type.equals("unknown"))
                 return getString(columnIndex);
 
+            if (type.equals("uuid"))
+                return getUUID(getString(columnIndex));
+
             // Specialized support for ref cursors is neater.
             if (type.equals("refcursor"))
             {
@@ -250,10 +254,7 @@ public abstract class AbstractJdbc2ResultSet implements BaseResultSet, org.postg
         }
 
         current_row = internalIndex;
-        this_row = (byte[][]) rows.elementAt(internalIndex);
-
-        rowBuffer = new byte[this_row.length][];
-        System.arraycopy(this_row, 0, rowBuffer, 0, this_row.length);
+        initRowBuffer();
         onInsertRow = false;
 
         return true;
@@ -295,10 +296,7 @@ public abstract class AbstractJdbc2ResultSet implements BaseResultSet, org.postg
             return false;
 
         current_row = 0;
-        this_row = (byte[][]) rows.elementAt(current_row);
-
-        rowBuffer = new byte[this_row.length][];
-        System.arraycopy(this_row, 0, rowBuffer, 0, this_row.length);
+        initRowBuffer();
         onInsertRow = false;
 
         return true;
@@ -629,10 +627,7 @@ public abstract class AbstractJdbc2ResultSet implements BaseResultSet, org.postg
             return false;
 
         current_row = rows_size - 1;
-        this_row = (byte[][]) rows.elementAt(current_row);
-
-        rowBuffer = new byte[this_row.length][];
-        System.arraycopy(this_row, 0, rowBuffer, 0, this_row.length);
+        initRowBuffer();
         onInsertRow = false;
 
         return true;
@@ -658,9 +653,7 @@ public abstract class AbstractJdbc2ResultSet implements BaseResultSet, org.postg
         {
             current_row--;
         }
-        this_row = (byte[][]) rows.elementAt(current_row);
-        rowBuffer = new byte[this_row.length][];
-        System.arraycopy(this_row, 0, rowBuffer, 0, this_row.length);
+        initRowBuffer();
         return true;
     }
 
@@ -877,10 +870,7 @@ public abstract class AbstractJdbc2ResultSet implements BaseResultSet, org.postg
         }
         else
         {
-            this_row = (byte[][]) rows.elementAt(current_row);
-
-            rowBuffer = new byte[this_row.length][];
-            System.arraycopy(this_row, 0, rowBuffer, 0, this_row.length);
+            initRowBuffer();
         }
 
         onInsertRow = false;
@@ -1140,7 +1130,7 @@ public abstract class AbstractJdbc2ResultSet implements BaseResultSet, org.postg
     throws SQLException
     {
         checkColumnIndex(columnIndex);
-        String columnTypeName = connection.getPGType(fields[columnIndex - 1].getOID());
+        String columnTypeName = connection.getTypeInfo().getPGType(fields[columnIndex - 1].getOID());
         updateValue(columnIndex, new NullObject(columnTypeName));
     }
 
@@ -1172,18 +1162,13 @@ public abstract class AbstractJdbc2ResultSet implements BaseResultSet, org.postg
 
         StringBuffer selectSQL = new StringBuffer( "select ");
 
-        final int numColumns = java.lang.reflect.Array.getLength(fields);
-
-        for (int i = 0; i < numColumns; i++ )
-        {
-            selectSQL.append( fields[i].getColumnName(connection) );
-
-            if ( i < numColumns - 1 )
-            {
-
+        ResultSetMetaData rsmd = getMetaData();
+        PGResultSetMetaData pgmd = (PGResultSetMetaData)rsmd;
+        for (int i=1; i <= rsmd.getColumnCount(); i++) {
+            if (i > 1) {
                 selectSQL.append(", ");
             }
-
+            selectSQL.append( pgmd.getBaseColumnName(i) );
         }
         selectSQL.append(" from " ).append(onlyTable).append(tableName).append(" where ");
 
@@ -1214,7 +1199,7 @@ public abstract class AbstractJdbc2ResultSet implements BaseResultSet, org.postg
 
         if ( rs.next() )
         {
-            rowBuffer = rs.rowBuffer;
+            rowBuffer = rs.this_row;
         }
 
         rows.setElementAt( rowBuffer, current_row );
@@ -1681,23 +1666,6 @@ public abstract class AbstractJdbc2ResultSet implements BaseResultSet, org.postg
                 switch ( getSQLType(columnIndex + 1) )
                 {
 
-                case Types.DECIMAL:
-                case Types.BIGINT:
-                case Types.DOUBLE:
-                case Types.BIT:
-                case Types.VARCHAR:
-                case Types.SMALLINT:
-                case Types.FLOAT:
-                case Types.INTEGER:
-                case Types.CHAR:
-                case Types.NUMERIC:
-                case Types.REAL:
-                case Types.TINYINT:
-		case Types.ARRAY:
-                case Types.OTHER:
-                    rowBuffer[columnIndex] = connection.encodeString(String.valueOf( valueObject));
-                    break;
-
                 //
                 // toString() isn't enough for date and time types; we must format it correctly
                 // or we won't be able to re-parse it.
@@ -1737,7 +1705,8 @@ public abstract class AbstractJdbc2ResultSet implements BaseResultSet, org.postg
                     break;
  
                 default:
-                    rowBuffer[columnIndex] = (byte[]) valueObject;
+                    rowBuffer[columnIndex] = connection.encodeString(String.valueOf( valueObject));
+                    break;
                 }
 
             }
@@ -1787,7 +1756,7 @@ public abstract class AbstractJdbc2ResultSet implements BaseResultSet, org.postg
 
     public String getRefCursor() {
         // Can't check this because the PGRefCursorResultSet
-	// interface doesn't allow throwing a SQLException
+        // interface doesn't allow throwing a SQLException
         //
         // checkClosed();
         return refCursorName;
@@ -1858,10 +1827,7 @@ public abstract class AbstractJdbc2ResultSet implements BaseResultSet, org.postg
             current_row++;
         }
 
-        this_row = (byte [][])rows.elementAt(current_row);
-
-        rowBuffer = new byte[this_row.length][];
-        System.arraycopy(this_row, 0, rowBuffer, 0, this_row.length);
+        initRowBuffer();
         return true;
     }
 
@@ -2151,6 +2117,71 @@ public abstract class AbstractJdbc2ResultSet implements BaseResultSet, org.postg
         return val;
     }
 
+    /**
+     * Optimised byte[] to number parser.  This code does not
+     * handle null values, so the caller must do checkResultSet
+     * and handle null values prior to calling this function.
+     * 
+     * @param columnIndex The column to parse.
+     * @return The parsed number.
+     * @throws SQLException If an error occurs while fetching column.
+     * @throws NumberFormatException If the number is invalid or the
+     * out of range for fast parsing. The value must then be parsed by
+     * {@link #toBigDecimal(String)}.
+     */
+    private BigDecimal getFastBigDecimal(int columnIndex) throws SQLException,
+        NumberFormatException {
+        
+        byte[] bytes = this_row[columnIndex - 1];
+        
+        if (bytes.length == 0) {
+            throw FAST_NUMBER_FAILED;
+        }
+        
+        int scale = 0;
+        long val = 0;
+        int start;
+        boolean neg;
+        if (bytes[0] == '-') {
+            neg = true;
+            start = 1;
+            if (bytes.length == 1 || bytes.length > 19) {
+                throw FAST_NUMBER_FAILED;
+            }
+        } else {
+            start = 0;
+            neg = false;
+            if (bytes.length > 18) {
+                throw FAST_NUMBER_FAILED;
+            }
+        }
+
+        int periodsSeen = 0;
+        while (start < bytes.length) {
+            byte b = bytes[start++];
+            if (b < '0' || b > '9') {
+                if (b == '.') {
+                    scale = bytes.length - start;
+                    periodsSeen++;
+                    continue;
+                } else
+                    throw FAST_NUMBER_FAILED;
+            }
+            val *= 10;
+            val += b - '0';
+        }
+
+        int numNonSignChars = neg ? bytes.length - 1 : bytes.length;
+        if (periodsSeen > 1 || periodsSeen == numNonSignChars)
+            throw FAST_NUMBER_FAILED;
+        
+        if (neg) {
+            val = -val;
+        }        
+        
+        return BigDecimal.valueOf(val, scale);
+    }
+
     public float getFloat(int columnIndex) throws SQLException
     {
         checkResultSet(columnIndex);
@@ -2175,6 +2206,14 @@ public abstract class AbstractJdbc2ResultSet implements BaseResultSet, org.postg
         if (wasNullFlag)
             return null;
         
+        Encoding encoding = connection.getEncoding();
+        if (encoding.hasAsciiNumbers()) {
+            try {
+                return getFastBigDecimal(columnIndex);
+            } catch (NumberFormatException ex) {
+            }
+        }
+        
         return toBigDecimal( getFixedString(columnIndex), scale );
     }
 
@@ -2587,12 +2626,12 @@ public abstract class AbstractJdbc2ResultSet implements BaseResultSet, org.postg
 
     protected String getPGType( int column ) throws SQLException
     {
-        return connection.getPGType(fields[column - 1].getOID());
+        return connection.getTypeInfo().getPGType(fields[column - 1].getOID());
     }
 
     protected int getSQLType( int column ) throws SQLException
     {
-        return connection.getSQLType(fields[column - 1].getOID());
+        return connection.getTypeInfo().getSQLType(fields[column - 1].getOID());
     }
 
     private void checkUpdateable() throws SQLException
@@ -2612,7 +2651,7 @@ public abstract class AbstractJdbc2ResultSet implements BaseResultSet, org.postg
 
     protected void checkClosed() throws SQLException {
         if (rows == null)
-            throw new PSQLException(GT.tr("This ResultSet is closed."), PSQLState.CONNECTION_DOES_NOT_EXIST);
+            throw new PSQLException(GT.tr("This ResultSet is closed."), PSQLState.OBJECT_NOT_IN_STATE);
     }
 
     protected void checkColumnIndex(int column) throws SQLException
@@ -2808,6 +2847,19 @@ public abstract class AbstractJdbc2ResultSet implements BaseResultSet, org.postg
         return 0;  // SQL NULL
     }
 
+    private void initRowBuffer()
+    {
+        this_row = (byte[][]) rows.elementAt(current_row);
+        // We only need a copy of the current row if we're going to
+        // modify it via an updatable resultset.
+        if (resultsetconcurrency == ResultSet.CONCUR_UPDATABLE) {
+            rowBuffer = new byte[this_row.length][];
+            System.arraycopy(this_row, 0, rowBuffer, 0, this_row.length);
+        } else {
+            rowBuffer = null;
+        }        
+    }
+    
     private boolean isColumnTrimmable(int columnIndex) throws SQLException
     {
         switch (getSQLType(columnIndex))
@@ -2865,10 +2917,21 @@ public abstract class AbstractJdbc2ResultSet implements BaseResultSet, org.postg
         checkColumnIndex(columnIndex);
 
         doingUpdates = !onInsertRow;
-        if (value == null)
+        if (value == null) {
             updateNull(columnIndex);
-        else
-            updateValues.put(fields[columnIndex - 1].getColumnName(connection), value);
+        } else {
+            PGResultSetMetaData md = (PGResultSetMetaData)getMetaData();
+            updateValues.put(md.getBaseColumnName(columnIndex), value);
+        }
+    }
+
+    /**
+     * Newer JVMs will return a java.util.UUID object, but it isn't
+     * available in older versions.
+     */
+    protected Object getUUID(String data) throws SQLException
+    {
+        return data;
     }
 
     private class PrimaryKey
@@ -2902,5 +2965,14 @@ public abstract class AbstractJdbc2ResultSet implements BaseResultSet, org.postg
             return null;
         }
     };
+
+    /**
+     * Used to add rows to an already existing ResultSet that exactly
+     * match the existing rows.  Currently only used for assembling
+     * generated keys from batch statement execution.
+     */
+    void addRows(Vector tuples) {
+        rows.addAll(tuples);
+    }
 }
 
