@@ -1,9 +1,9 @@
 /*-------------------------------------------------------------------------
 *
-* Copyright (c) 2004-2005, PostgreSQL Global Development Group
+* Copyright (c) 2004-2011, PostgreSQL Global Development Group
 *
 * IDENTIFICATION
-*   $PostgreSQL: pgjdbc/org/postgresql/jdbc4/Jdbc4Blob.java,v 1.1.2.1 2007/02/19 06:04:48 jurka Exp $
+*   $PostgreSQL: pgjdbc/org/postgresql/jdbc4/Jdbc4Blob.java,v 1.5 2011/08/02 13:49:23 davecramer Exp $
 *
 *-------------------------------------------------------------------------
 */
@@ -15,7 +15,7 @@ import java.sql.*;
 public class Jdbc4Blob extends AbstractJdbc4Blob implements java.sql.Blob
 {
 
-    public Jdbc4Blob(org.postgresql.PGConnection conn, long oid) throws SQLException
+    public Jdbc4Blob(org.postgresql.core.BaseConnection conn, long oid) throws SQLException
     {
         super(conn, oid);
     }
